@@ -54,6 +54,7 @@
 
 static char batteries[MAXBATT][128];
 static char battinfo[MAXBATT][128];
+#ifndef __linux__
 #if HAVE_SYSCTL
  static int
 name2oid(char *name, int *oidp)
@@ -177,6 +178,7 @@ get_var(int *oid, int nlen)
 }
 
  
+#endif
 #endif
 
 /* see if we have ACPI support */
@@ -277,6 +279,9 @@ int read_acad_state(void)
     if (!(acpi = fopen ("/proc/acpi/ac_adapter/ACAD/state", "r")))
       if (!(acpi = fopen ("/proc/acpi/ac_adapter/AC/state", "r")))
         if (!(acpi = fopen ("/proc/acpi/ac_adapter/ADP1/state", "r")))
+         if (!(acpi = fopen ("/proc/acpi/ac_adapter/AC0/state", "r")))
+	    if (!(acpi = fopen ("/proc/acpi/ac_adapter/ADP0/state", "r")))
+	      if (!(acpi = fopen ("/proc/acpi/ac_adapter/C11B/state", "r")))
 	        return -1;
 
   fread (buf, 512, 1, acpi);
@@ -318,6 +323,7 @@ int read_acad_state(void)
   int retval;
   snprintf(buf, BUFSIZ, "%s", "hw.acpi.acline");
   len = name2oid(bufp, mib);
+  if (len <= 0) return(-1);
   if (oidfmt(mib, len, fmt, &kind))
 	err(1, "couldn't find format of oid '%s'", bufp);
   if (len < 0) errx(1, "unknown oid '%s'", bufp);
@@ -475,6 +481,7 @@ int read_acpi_info(int battery)
 	    acpiinfo->design_capacity_low = 0;
   snprintf(buf, BUFSIZ, "%s", "hw.acpi.battery.units");
   len = name2oid(bufp, mib);
+  if (len <= 0) return(-1);
   if (oidfmt(mib, len, fmt, &kind))
 	err(1, "couldn't find format of oid '%s'", bufp);
   if (len < 0) errx(1, "unknown oid '%s'", bufp);
@@ -623,6 +630,7 @@ int read_acpi_state(int battery)
   
   snprintf(buf, BUFSIZ, "%s", "hw.acpi.battery.time");
   len = name2oid(bufp, mib);
+  if (len <= 0) return(-1);
   if (oidfmt(mib, len, fmt, &kind))
 	err(1, "couldn't find format of oid '%s'", bufp);
   if (len < 0) errx(1, "unknown oid '%s'", bufp);
@@ -638,6 +646,7 @@ int read_acpi_state(int battery)
   
   snprintf(buf, BUFSIZ, "%s", "hw.acpi.battery.life");
   len = name2oid(bufp, mib);
+  if (len <= 0) return(-1);
   if (oidfmt(mib, len, fmt, &kind))
 	err(1, "couldn't find format of oid '%s'", bufp);
   if (len < 0) errx(1, "unknown oid '%s'", bufp);
@@ -710,6 +719,7 @@ const char *get_temperature(void)
   u_int kind;
   snprintf(buf, BUFSIZ, "%s", "hw.acpi.thermal.tz0.temperature");
   len = name2oid(bufp, mib);
+  if (len <= 0) return(NULL);
   if (oidfmt(mib, len, fmt, &kind))
 	err(1, "couldn't find format of oid '%s'", bufp);
   if (len < 0) errx(1, "unknown oid '%s'", bufp);
